import OHeader from '@/components/o-header'
import OSearch from '@/components/o-search'
import OSticky from '@/components/o-sticky'
import OEmpty from '@/components/o-empty'
import dayjs from 'dayjs'
import {
  Cell,
  Icon,
  Popover,
  Tag,
  DatePicker,
  DatePickerColumnType,
  Popup,
  List,
  PullRefresh,
  ActionSheet,
  showToast,
  Picker
} from 'vant'
import OFullRefresh from '@/components/o-full-refresh'

import StudentItem from './modals/student-item'
import { defineComponent, reactive, ref, onMounted } from 'vue'
import { state as globalState } from '@/state'
import { useRouter } from 'vue-router'
import styles from './index.module.less'
import request from '@/helpers/request'
import OActionSheet from '@/components/o-action-sheet'
import { formatterDatePicker } from '@/helpers/utils'
export default defineComponent({
  name: 'exercise-record',
  setup() {
    const platformApi = ref(globalState.platformApi)
    const router = useRouter()
    const state = reactive({
      showPopoverTime: false,
      showPopoverOrchestra: false,
      showPopoverSubject: false,
      showPopoverSort: false,
      actions: [] as any,
      subjects: [] as any,
      actionSorts: [
        {
          name: '按天数',
          value: 'PRACTICE_DAY',
          selected: true
        },
        {
          name: '按时长',
          value: 'PRACTICE_TIMES'
        }
        //   color: forms.sortType == 'PRACTICE_DAY' ? '#FF8057' : '#333'
      ],
      currentDate: [dayjs().format('YYYY'), dayjs().format('MM')]
    })
    const forms = reactive({
      practiceMonth: state.currentDate[0] + '' + state.currentDate[1],
      practiceMonthName: state.currentDate[0] + '年' + state.currentDate[1] + '月',
      orchestraId: '',
      orchestraName: '全部乐团',
      subjectId: '',
      subjectName: '全部声部',
      sortType: 'PRACTICE_DAY',
      sortTypeName: '按天数',
      keyword: '',
      page: 1,
      rows: 20
    })
    const minDate = ref(new Date(dayjs().subtract(10, 'year').format('YYYY-MM-DD')))
    const maxDate = ref(new Date(dayjs().add(10, 'year').format('YYYY-MM-DD')))
    const columnsType = ref<DatePickerColumnType[]>(['year', 'month'])
    const refreshing = ref(false)
    const loading = ref(false)
    const finished = ref(false)
    const showContact = ref(false)
    const list = ref([])
    const getList = async () => {
      loading.value = true
      try {
        if (refreshing.value) {
          forms.page = 1
          list.value = []
          refreshing.value = false
        }
        const res = await request.post(`${platformApi.value}/student/page`, {
          data: { ...forms }
        })

        if (list.value.length > 0 && res.data.pages === 1) {
          return
        }

        forms.page = res.data.current + 1
        list.value = list.value.concat(res.data.rows || [])
        showContact.value = list.value.length > 0
        console.log(showContact.value, ' showContact.value ')
        loading.value = false

        finished.value = res.data.current >= res.data.pages
      } catch (e: any) {
        // console.log(e, 'e')
        const message = e.message
        showToast(message)
        showContact.value = false
        finished.value = true
      }
    }
    onMounted(() => {
      getList()
      getOrchestraList()
      getSubjects()
    })

    const onBack = () => {
      router.go(-1)
    }

    const checkSort = (val: any) => {
      forms.sortType = val.value
      forms.sortTypeName = val.name
      state.actionSorts.forEach((child: any) => {
        child.selected = false
      })
      val.selected = true
      state.showPopoverSort = false
      refreshing.value = true
      getList()
    }
    const checkTimer = (val: any) => {
      forms.practiceMonth = val.selectedValues[0] + val.selectedValues[1]
      forms.practiceMonthName = val.selectedValues[0] + '年' + val.selectedValues[1] + '月'
      state.showPopoverTime = false
      refreshing.value = true
      getList()
    }
    const checkOrchestra = (val: any) => {
      const selectedOptions = val.selectedOptions[0] || {}
      forms.orchestraId = selectedOptions.value
      forms.orchestraName = selectedOptions.name
      state.showPopoverOrchestra = false
      refreshing.value = true
      getList()
    }

    const checkSubject = (val: any) => {
      const selectedOptions = val.selectedOptions[0] || {}
      forms.subjectId = selectedOptions.value
      forms.subjectName = selectedOptions.name
      state.showPopoverSubject = false
      refreshing.value = true
      getList()
    }
    const getOrchestraList = async () => {
      try {
        const res = await request.post(`${platformApi.value}/orchestra/page`, {
          data: { page: 1, rows: 9999, status: 'DONE' }
        })
        state.actions = res.data.rows.map((item) => {
          return {
            name: item.name,
            value: item.id as string
          }
        })
        state.actions.unshift({ name: '全部乐团', value: '' })
      } catch (e: any) {
        const message = e.message
        showToast(message)
      }
    }

    const getSubjects = async () => {
      try {
        const res = await request.post(`${platformApi.value}/subjectBasicConfig/page`, {
          data: { page: 1, rows: 9999 }
        })
        state.subjects = res.data.rows.map((item) => {
          return {
            name: item.subjectName,
            value: item.subjectId as string
          }
        })
        state.subjects.unshift({ name: '全部声部', value: '' })
      } catch (e: any) {
        const message = e.message
        showToast(message)
      }
    }
    const onRefresh = () => {
      finished.value = false
      // 重新加载数据
      // 将 loading 设置为 true，表示处于加载状态
      loading.value = true
      getList()
    }

    return () => (
      <div class={!showContact.value && 'emptyRootContainer'}>
        <OSticky position="top" background="#F8F8F8">
          <div>
            {/* <OHeader onHeaderBack={onBack}></OHeader> */}
            <OSearch
              placeholder="请输入学生姓名"
              class={styles.recordSearch}
              onSearch={(val: any) => {
                forms.keyword = val
                refreshing.value = true
                getList()
              }}
            ></OSearch>
            <div class={'searchGroup'}>
              <div
                class={['searchItem', state.showPopoverTime ? 'searchItem-active' : '']}
                onClick={() => {
                  state.showPopoverTime = true
                }}
              >
                <span>{forms.practiceMonthName}</span>
                <i class="arrow"></i>
              </div>
              <div
                class={['searchItem', state.showPopoverOrchestra ? 'searchItem-active' : '']}
                onClick={() => {
                  state.showPopoverOrchestra = true
                }}
              >
                <span>{forms.orchestraName}</span>
                <i class="arrow"></i>
              </div>
              <div
                class={['searchItem', state.showPopoverSubject ? 'searchItem-active' : '']}
                onClick={() => {
                  state.showPopoverSubject = true
                }}
              >
                <span>{forms.subjectName}</span>
                <i class="arrow"></i>
              </div>
              <div
                class={['searchItem', state.showPopoverSort ? 'searchItem-active' : '']}
                onClick={() => {
                  state.showPopoverSort = true
                }}
              >
                <span>{forms.sortTypeName}</span>
                <i class="arrow"></i>
              </div>
            </div>
          </div>
        </OSticky>
        {showContact.value ? (
          <OFullRefresh
            v-model:modelValue={refreshing.value}
            onRefresh={onRefresh}
            style="min-height: 100vh;"
          >
            <List
              loading-text=" "
              // v-model:loading={loading.value}
              finished={finished.value}
              style={{
                paddingTop: '12px'
              }}
              finished-text="没有更多了"
              onLoad={getList}
            >
              {list.value.map((item: any) => (
                <StudentItem item={item} forms={forms} />
              ))}
            </List>
          </OFullRefresh>
        ) : (
          <OEmpty tips="暂无练习记录" />
        )}

        <Popup
          v-model:show={state.showPopoverTime}
          position="bottom"
          round
          class={'popupBottomSearch'}
        >
          <DatePicker
            onCancel={() => {
              state.showPopoverTime = false
            }}
            onConfirm={checkTimer}
            v-model={state.currentDate}
            minDate={minDate.value}
            formatter={formatterDatePicker}
            maxDate={maxDate.value}
            columnsType={columnsType.value}
          />
        </Popup>

        <OActionSheet
          v-model:show={state.showPopoverSort}
          actions={state.actionSorts}
          onSelect={checkSort}
        />

        <Popup
          v-model:show={state.showPopoverOrchestra}
          position="bottom"
          round
          class={'popupBottomSearch'}
        >
          <Picker
            columns={state.actions}
            onCancel={() => (state.showPopoverOrchestra = false)}
            onConfirm={(val: any) => checkOrchestra(val)}
            columnsFieldNames={{ text: 'name', value: 'value' }}
          />
        </Popup>

        <Popup
          v-model:show={state.showPopoverSubject}
          position="bottom"
          round
          class={'popupBottomSearch'}
        >
          <Picker
            columns={state.subjects}
            onCancel={() => (state.showPopoverSubject = false)}
            onConfirm={(val: any) => checkSubject(val)}
            columnsFieldNames={{ text: 'name', value: 'value' }}
          />
        </Popup>
      </div>
    )
  }
})
